function [R,Rpun,tm,etaGrados,s,rho,rhodot] = AdmissibleRegion(Datos,opc,mm)
tic
%     %This is the main function corresponding to the scientific article
%     %"Determination of the admissible region of asteroids with data from
%     %one night of observation".
%     
% %The input data "Datos" correspond to a matrix where    
% %Datos(:,1): Times in Juliana date
% %Datos(:,2): Right Ascension in radians
% %Datos(:,3): Declination in radians
% %opc: Opcion 1 Geocentric, 2 topocentric
% %mm: metric 1 Logarimic 2 Exponential

% %Output data
% R,Rpun: state vector of the Earth
% tm: mean time
% etaGrados: proper motion
% s: maximum root of 6th degree polynomial
% rho: range data of the AR
% rhodot: range-rate data of the AR

close all
format longg
% Numerical constants along the code
k = 0.01720209895;   %Gaussian gravitational constant 
RSI = 0.010044;      %radius of influence of the Earth
miu = 1/328900.5614; %ratio mass of the Earth/the mass of the Sun
Rtierra = 4.24e-05;  %Radius of the Earth
 
if opc==1
    fprintf('\n\n******No Topocentric correction is applied\n')
else
    fprintf('    \nTopocentric correction is applied\n')
end

coe = attributable(Datos,opc);
alpha = coe(1);
alphap = coe(2);
delta = coe(3);
deltap = coe(4);
R = coe(5:7);
Rpun = coe(8:10);
tm = mean(Datos(:,1));  %mean time of initial observations

% Define the inner boundary
switch opc
    case 1 %No se aplicara correcin topocentrica
        LimInter = Rtierra;
        interior = 1; 
    case 2 %If you use topocentric correccion, you must know the value of h
        h = 20; %average of apparent magnitudes, example for 2003 BH84
        Hmax = 22; %maximum value of absolute magnitude example for 2003 BH84
        ro_H = 10^((h-Hmax)/5);
        LimInter = ro_H;
        if LimInter>RSI   %condition where the geometry is more easy
            interior = 0;   
        else
            interior = 1; 
        end
end

% Computing the orthonormal basis ro, ro_alfa, ro_delta
ro_g = [cos(alpha)*cos(delta) sin(alpha)*cos(delta) sin(delta)];  
roalpha_g = [-1*sin(alpha)*cos(delta) cos(alpha)*cos(delta) 0];
rodelta_g = [-1*cos(alpha)*sin(delta) -1*sin(alpha)*sin(delta) cos(delta)];
% 3.proper motion squared
etac = alphap*alphap*cos(delta)*cos(delta) + deltap^2;  %para convertir a grados usar rad2deg
eta=sqrt(etac);
etaGrados =  round(rad2deg(eta),4); 

% The objetc belong to the Solar System
% Computing of the coefficients
e0 = norm(R)^2;
e1 = 2*dot(Rpun,ro_g);
e2 = alphap^2*(cos(delta))^2 + deltap^2;
e3 = 2*alphap*dot(Rpun,roalpha_g) + 2*deltap*dot(Rpun,rodelta_g);
e4 = norm(Rpun)^2;
e5 = 2*dot(R,ro_g);

%Compute the gamma value
switch opc
    case 1 % topocentric correction not applied
        Emenor = 0;
        gamma = e4 - (e1^2)/4; % gamma for E=0;
    case 2 % topocentric correction applied
        gamma = e4 - (e1^2)/4 + k^2/100; %gama modified to exclude Long T comets
        Emenor = k^2/100;
end

% Computing the omega coefficients for the 6th degree polynomial
w0 = e0*gamma^2;
w1 = e5*gamma^2 + 2*e0*e3*gamma;
w2 = gamma^2 + 2*e3*e5*gamma + e0*(e3^2 + 2*e2*gamma);
w3 = 2*e3*gamma + e5*(e3^2 + 2*e2*gamma) + 2*e0*e2*e3;
w4 = e3^2 + 2*e2*gamma + 2*e2*e3*e5 + e0*e2^2;
w5 = e2*(2*e3 + e2*e5);
w6 = e2^2;

% Roots of the polynomial V(p)-4k^4;
Ai = [w6 w5 w4 w3 w2 w1 w0-4*(k^4)];
raices = roots(Ai);
raicesReales=real(raices);
s = max(raicesReales);
% Warning message for more than two comp. connected
 cont=0;
for i=1:length(raices)
    fase = imag(raices(i))/real(raices(i));
    if fase == 0
        cont = cont+1;
    end
end
if cont>=3
    fprintf('    \nThere are two connected components, it is a distant asteroid\n')
else
    fprintf('    \nThere one connected components\n')
end
% Compute the Energy level value
digits(128)
ro_maximo = s;
ro = LimInter:ro_maximo/(1000):ro_maximo; %100000
ro_maximo_Redon = round(ro_maximo,4)
ro2 = linspace(LimInter,ro_maximo_Redon,1000);
a=max(ro2);
a1 = length(ro);
b=ro(end);
b1 = length(ro2);
ro=ro2;
 
ropdaEH_p = zeros(size(ro)); %Positive part of the heliocentric energy curve
ropdaEH_n = zeros(size(ro)); %Negative part of the heliocentric energy curve
% This variable stores the positions where imaginary values are presented
compl=[];
aux=0; % initially there is nothing of imaginary roots
for i=1:length(ro)
    Sro = ro(i)^2 + e5*ro(i) + e0;
    Wro = e2*ro(i)^2 + e3*ro(i) + e4;
    discr = (e1^2)/4 - Wro + (2*k^2)/sqrt(Sro)- Emenor; %discriminant for E<=0  E<=-k2/2*a
    if discr<0
        aux=1;    %indicates if there are complex numbers
        compl = [compl,i];
    end
    rop_p = -e1/2 + sqrt(discr);  %Upper region
    rop_n = -e1/2 - sqrt(discr);  %Lower region
    ropdaEH_p(i) = rop_p;
    ropdaEH_n(i) = rop_n;
end

% The complex solutions that are presented at the end are discarded,
% so that the end of the plot approaches the optimal results
if(aux==1) %Were complex numbers presented?
    ropdaEH_p = ropdaEH_p(1:compl(1));
    ropdaEH_n = ropdaEH_n(1:compl(1));
    ro = ro(1:compl(1));
    ropdaEH_p(end)=real(ropdaEH_p(end));
    ropdaEH_n(end)=real(ropdaEH_n(end));
end

    function rom = metrica(posc,metri)   
        %This function changes the metric
        %output: rom-> ro modified
        if metri ==1
            rom = log10(posc);
        else
            rom = 1-exp(-(posc.^2)/(2*(s)^2));
        end
        
    end
rom=metrica(ro,mm);
max(ro)  
figure
plot(rom,ropdaEH_p,rom,ropdaEH_n,'LineWidth',1.5)
xlabel('f(\rho)  \rho in AU','FontUnits','points','interpreter',...
    'tex','FontSize',18,'FontName','Times')
ylabel('d\rho/dt,  (AU/day)','FontUnits','points','interpreter',...
    'tex','FontSize',18,'FontName','Times')
title('Admissible Region','FontSize',20,'FontName','Times')

%*********
%%%%%%%% If you know the real position of the small bodie uncoment
aaa = 1.992810307469066E+00;    %Example for 2003 BH84
bbb = 6.522301101026108E-03;
X = metrica(aaa,mm);% ** 1.993093574857729E+00;
Y = bbb ;
hold on
plot(X,Y,'o','MarkerSize',6,'MarkerEdgeColor','red','MarkerFaceColor',[1 .6 .6])
%%%%%%%%

% Compute the value of rho_0
ro_cero = ((2*k^2*miu)/etac)^(1/3);

% Region of exclusion of satellites of the Earth
ropda = [];   %ro_posicion data
ropdan = [];  %ro_posicion data for negative part
switch interior
    case 0
        ph_punto = linspace(ropdaEH_n(1),ropdaEH_p(1),20); 
        ph = zeros(size(ph_punto));
        for i=1:length(ph_punto)
            ph(i)=LimInter;
        end
        hold on
        ph = metrica(ph,mm);
        plot(ph,ph_punto,'Color','red','LineStyle','-.','LineWidth',1.5)
        rho = [rom, fliplr(rom),ph];
        rhodot = [ropdaEH_n, fliplr(ropdaEH_p),ph_punto];
    case 1
        %It is done if rho_H is less than RSI or if there is no topocentric correction
        if ro_cero > RSI   %Is the estimator greater than the radius of influence?
            roGeo = LimInter:RSI/(1000):RSI;
            
            for i=1:length(roGeo)
                rop = sqrt((2*k^2*miu)/roGeo(i) -etac*roGeo(i)^2);
                ropn = -1*rop;
                ropda = [ropda,rop];   %ro punto data positive part
                ropdan = [ropdan,ropn];   %ro punto data negative part
            end

            hold on
            roGeo = metrica(roGeo,mm);
            plot(roGeo,ropda,roGeo,ropdan,'LineWidth',1.5) 
            % option 1: physical radio of the Earth or inner boundary
            % This line is divided into two parts: Portion 1:
            [LimiteInte_vecP1,rhopLimiInteP1] = meshgrid(linspace(LimInter...
                ,LimInter,length(ropda)/2),ropdaEH_n(1):(ropdan(1)-...
                ropdaEH_n(1))/10:ropdan(1));
            [LimiteInte_vecP2,rhopLimiInteP2] = meshgrid(linspace(LimInter...
                ,LimInter,length(ropda)/2),ropda(1):(ropdaEH_p(1)-...
                ropda(1))/10:ropdaEH_p(1));
            hold on
          
            LimiteInte_vecP1 = metrica(LimiteInte_vecP1,mm);
            LimiteInte_vecP2 = metrica(LimiteInte_vecP2,mm);
            plot(LimiteInte_vecP1,rhopLimiInteP1,LimiteInte_vecP2,...
                rhopLimiInteP2,'Color','red','LineWidth',1.5)
            aux1 = filas(LimiteInte_vecP1)';
            aux2= filas(LimiteInte_vecP2)';
            aux3 = filas(rhopLimiInteP1)';
            aux4= filas(rhopLimiInteP2)';
            
            hold on
            [RSI_vec,rhopRSI] = meshgrid(linspace(RSI,RSI,15),ropdan(end)...
                :(ropda(end)-ropdan(end))/10:ropda(end));
            RSI_vec= metrica(RSI_vec,mm);
            plot(RSI_vec,rhopRSI,'Color','red','LineWidth',1.5) 
            vu = filas(RSI_vec)';
            vd = filas(rhopRSI)';
            rho = [aux2,rom,fliplr(rom),aux1,roGeo,vu,fliplr(roGeo)];
            rhodot = [aux4,ropdaEH_p,fliplr(ropdaEH_n),aux3,ropdan,vd,fliplr(ropda)];
        else
            % Enter when rho_cero is less than or equal to the radius
            % of influence of the Earth
            roRtierra = LimInter:(ro_cero-LimInter)/(1000):ro_cero;
            for i=1:length(roRtierra)
                rop = sqrt((2*k^2*miu)/roRtierra(i) -etac*roRtierra(i)^2);
                ropda = [ropda,rop];   %drho/dt data
                ropn = -1*rop;
                ropdan = [ropdan,ropn];   %drho/dt data negative
            end
            
            roRtierra = metrica(roRtierra,mm);
            plot(roRtierra,ropda,roRtierra,ropdan,'LineWidth',1.5)            
            [LimiteInte_vecP1,rhopLimiInteP1] = meshgrid(linspace(LimInter,...
                LimInter,length(ropda)/2),ropdaEH_n(1):(ropdan(1)-...
                ropdaEH_n(1))/10:ropdan(1)); 
            [LimiteInte_vecP2,rhopLimiInteP2] = meshgrid(linspace(LimInter,...
                LimInter,length(ropda)/2),ropda(1):(ropdaEH_p(1)-...
                ropda(1))/10:ropdaEH_p(1));
            
            hold on
            LimiteInte_vecP1 = metrica(LimiteInte_vecP1,mm);
            LimiteInte_vecP2 = metrica(LimiteInte_vecP2,mm);
            plot(LimiteInte_vecP1,rhopLimiInteP1,LimiteInte_vecP2,...
                rhopLimiInteP2,'Color','red','LineWidth',1.5)
            aux1 = filas(LimiteInte_vecP1)';
            aux2= filas(LimiteInte_vecP2)';
            aux3 = filas(rhopLimiInteP1)';
            aux4= filas(rhopLimiInteP2)';
            rho = [aux2,rom,fliplr(rom),aux1,roRtierra,fliplr(roRtierra)];
            rhodot = [aux4,ropdaEH_p,fliplr(ropdaEH_n),aux3,ropdan,fliplr(ropda)];
        end
%         At this point the conditions are satisfied
%         At this point the conditions are satisfied 

end
%         If you want to generate a better plot in eps, uncomment this function
% EPSPlotAR(-4.5,1,-0.016,0.022,opc) %see their properties example for geocentric and logarithmic
    function v = filas(vec)
        f= size(vec);
        v = [];
        for i=1:f(1)
            v = [v;vec(i,:)'];
        end
    end
end